@@ -51,6 +51,7 @@
 #define NETDATA_CHART_PRIO_SYSTEM_INTERRUPTS           1000
 #define NETDATA_CHART_PRIO_SYSTEM_DEV_INTR             1000 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_SOFT_INTR            1100 // freebsd only
+#define NETDATA_CHART_PRIO_SYSTEM_FILES_NR             1000
 #define NETDATA_CHART_PRIO_SYSTEM_ENTROPY              1000
 #define NETDATA_CHART_PRIO_SYSTEM_UPTIME               1000
 #define NETDATA_CHART_PRIO_CLOCK_SYNC_STATE            1100
@@ -395,16 +396,17 @@
 #define ML_CHART_PRIO_DETECTOR_EVENTS   39183
 
 // [netdata.ml] charts
-#define NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS 890001
-#define NETDATA_ML_CHART_PRIO_METRIC_TYPES            890002
-#define NETDATA_ML_CHART_PRIO_TRAINING_STATUS         890003
+#define NETDATA_ML_CHART_RUNNING                      890001
+#define NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS 890002
+#define NETDATA_ML_CHART_PRIO_METRIC_TYPES            890003
+#define NETDATA_ML_CHART_PRIO_TRAINING_STATUS         890004
 
-#define NETDATA_ML_CHART_PRIO_PREDICTION_USAGE        890004
-#define NETDATA_ML_CHART_PRIO_TRAINING_USAGE          890005
+#define NETDATA_ML_CHART_PRIO_PREDICTION_USAGE        890005
+#define NETDATA_ML_CHART_PRIO_TRAINING_USAGE          890006
 
-#define NETDATA_ML_CHART_PRIO_QUEUE_STATS             890006
-#define NETDATA_ML_CHART_PRIO_TRAINING_TIME_STATS     890007
-#define NETDATA_ML_CHART_PRIO_TRAINING_RESULTS        890008
+#define NETDATA_ML_CHART_PRIO_QUEUE_STATS             890007
+#define NETDATA_ML_CHART_PRIO_TRAINING_TIME_STATS     890008
+#define NETDATA_ML_CHART_PRIO_TRAINING_RESULTS        890009
 
 #define NETDATA_ML_CHART_FAMILY "machine learning"
 #define NETDATA_ML_PLUGIN "ml.plugin"
