@@ -51,6 +51,26 @@ typedef enum rrdr_options {
     RRDR_OPTION_INTERNAL_AR          = (1 << 31), // internal use only, to let the formatters know we want to render the anomaly rate
 } RRDR_OPTIONS;
 
+typedef enum context_v2_options {
+    CONTEXT_V2_OPTION_MINIFY                        = (1 << 0), // remove JSON spaces and newlines from JSON output
+    CONTEXT_V2_OPTION_DEBUG                         = (1 << 1), // show the request
+    CONTEXT_V2_OPTION_ALERTS_WITH_CONFIGURATIONS    = (1 << 2), // include alert configurations (used by /api/v2/alert_transitions)
+    CONTEXT_V2_OPTION_ALERTS_WITH_INSTANCES         = (1 << 3), // include alert instances      (used by /api/v2/alerts)
+    CONTEXT_V2_OPTION_ALERTS_WITH_VALUES            = (1 << 4), // include alert latest values  (used by /api/v2/alerts)
+    CONTEXT_V2_OPTION_ALERTS_WITH_SUMMARY           = (1 << 5), // include alerts summary counters  (used by /api/v2/alerts)
+} CONTEXTS_V2_OPTIONS;
+
+typedef enum context_v2_alert_status {
+    CONTEXT_V2_ALERT_UNINITIALIZED  = (1 << 5), // include UNINITIALIZED alerts
+    CONTEXT_V2_ALERT_UNDEFINED      = (1 << 6), // include UNDEFINED alerts
+    CONTEXT_V2_ALERT_CLEAR          = (1 << 7), // include CLEAR alerts
+    CONTEXT_V2_ALERT_RAISED         = (1 << 8), // include WARNING & CRITICAL alerts
+    CONTEXT_V2_ALERT_WARNING        = (1 << 9), // include WARNING alerts
+    CONTEXT_V2_ALERT_CRITICAL       = (1 << 10), // include CRITICAL alerts
+} CONTEXTS_V2_ALERT_STATUS;
+
+#define CONTEXTS_V2_ALERT_STATUSES (CONTEXT_V2_ALERT_UNINITIALIZED|CONTEXT_V2_ALERT_UNDEFINED|CONTEXT_V2_ALERT_CLEAR|CONTEXT_V2_ALERT_RAISED|CONTEXT_V2_ALERT_WARNING|CONTEXT_V2_ALERT_CRITICAL)
+
 typedef enum __attribute__ ((__packed__)) rrdr_value_flag {
 
     // IMPORTANT:
